// GPL'd - Full license at http://www.gnu.org/copyleft/gpl.html
// Copyright (c) 2002 Matthew Wlazlo <dhairrhuwyth@yahoo.com>
// 
#include <Imlib.h>
#include <X11/Xlib.h>
#include <ctype.h>
#include <assert.h>
#include <errno.h>

typedef struct _flist {
	struct _flist *next;
	char *name;
	FILE *fp;
} flist;

Display *gDisp;
ImlibData *gImlib;
unsigned char *gData;
ImlibImage *gImage;
flist *gFiles = 0, *gCurFile = 0;


// Things I have to know how to output:
// 	< and >
// 	newlines
void PrintPixel(unsigned char r, unsigned char g, unsigned char b, char c)
{
	printf("<font color=\"%x%x%x\">", r, g, b);
	if(c == '<')
		printf("&lt;");
	else if(c == '>')
		printf("&gt;");
	else if(c == ' ')
		printf("&nbsp;");
	else
		printf("%c", c);
	printf("</font>");
	fflush(stdout);
}

char NextChar()
{
	assert(gCurFile != 0);
	assert(gFiles != 0);
	char ret = 0;
	static char last = -1;
	int v;

	v = fread(&ret, 1, 1, gCurFile->fp);

	if(v <= 0)
		if(feof(gCurFile->fp)) {
			fprintf(stderr, "Finished with %s\n", gCurFile->name);
									
			clearerr(gCurFile->fp);
			rewind(gCurFile->fp);
			if(gCurFile->next == 0) {
				gCurFile = gFiles;
			} else {
				gCurFile = gCurFile->next;
			}
			fprintf(stderr, "Using %s\n", gCurFile->name);
			return NextChar();
		} else {
			fprintf(stderr, "[fread] %s: %s\n", gCurFile->name, strerror(ferror(gCurFile->fp)));
			exit(1);
		}

	assert(ret != 0);

	// skip past >2 spaces
	if(isspace(ret))
		ret = ' ';

	if(ret == ' ')
		return NextChar();
	else {
		last = ret;
		return ret;
	}
}

void ReadPixels()
{
	unsigned char r, g, b, c;
	int x, y;

	for(y = 0; y < gImage->rgb_height; y++) {
		for(x = 0; x < gImage->rgb_width; x++) {
			c = NextChar();
			r = gData[0];
			g = gData[1];
			b = gData[2];
			gData += 3;
			PrintPixel(r, g, b, c);
		}
		printf("<br>\n");
	}
	
}

int main(int argc, char **argv)
{
	if(argc < 3) {
		fprintf(stderr, "Usage: img2html image.[png|gif|jpeg] text\n");
		return 1;
	}

	gDisp = XOpenDisplay(NULL);
	
	gImlib = Imlib_init(gDisp);
	gImage = Imlib_load_image(gImlib, argv[1]);
	if(!gImage) {
		fprintf(stderr, "Error loading image\n");
		return 1;
	}
	gData = gImage->rgb_data;

	// put all the files into a list...
	flist *list = 0;
	int i = 2;
	while(argv[i] != 0) {
		flist *n;
		FILE *fp = fopen(argv[i],"r");
		if(!fp) {
			fprintf(stderr, "[fopen] %s: %s\n", argv[i], strerror(errno));
			return 1;
		}

		n = new flist;
		n->next = 0;
		n->fp = fp;
		n->name = argv[i];
		
		if(!list) {
			list = n;
			gFiles = list;
			gCurFile = gFiles;
		} else {
			flist *iter = list;
			while(iter->next != 0)
				iter = iter->next;
			assert(iter);
			iter->next = n;
		}
		i++;
	}

	printf("<html><head><style>\n");
	printf("body { background: black; }\n");
	printf("tt { font-size: 9pt; }\n");
	printf("</style></head><body>\n");
	printf("<tt>\n");
	ReadPixels();
	printf("</tt>\n");
	printf("</body>\n");
	
	// cleanup...
	flist *iter = gFiles;
	while(iter) {
		fclose(iter->fp);
		flist *gonner = iter;
		iter = iter->next;
		delete gonner;
	}

	fprintf(stderr, "Exiting normally\n");
	return 0;
}

// vim: ts=8 
